/*
  XMPlay MSN Plugin (c) 2005 Elliott Sales de Andrade

Copyright (c) 2005 Elliott Sales de Andrade

This software is provided 'as-is', without any express or implied warranty.
In no event will the authors be held liable for any damages arising from the
use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it freely,
subject to the following restrictions:

    1. The origin of this software must not be misrepresented; you must not
       claim that you wrote the original software. If you use this software
       in a product, an acknowledgment in the product documentation would be
       appreciated but is not required.

    2. Altered source versions must be plainly marked as such, and must not
       be misrepresented as being the original software.

    3. This notice may not be removed or altered from any source distribution.

Update 2, Elliott Sales de Andrade
  - Incorporate Svante's changes to my copy
  - Stop showing "XMPlay 3.2"
  - Mini config
  - Source rearrangements
Update 1, Svante Boberg
  - Fixed crash when trying to open configuration
  - Safer release of subclass, sometimes crashed the player in last version
  - Automatically sets MSN now playing when loading plugin
*/

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <memory.h>

#include "xmpdsp.h"
#include "xmp-msn.h"

static HINSTANCE ghInstance;
static WNDPROC lpPrevWndProc;
static COPYDATASTRUCT msndata;

// config structure
typedef struct {
	BOOL showXMP;
	BOOL keepOnClose;
} MSNStuff;
static MSNStuff msnConf;

static XMPDSP dsp = {
    0,
    0,
    "XMPlay2MSN",
    DSP_About,
    DSP_New,
    DSP_Free,
    DSP_GetDescription,
    DSP_Config,
    DSP_GetConfig,
    DSP_SetConfig,
    DSP_NewTrack,
    DSP_SetFormat,
    DSP_Reset,
    DSP_Process
};

// msvc was being annoying about sscanf, so I had to write this...
static BOOL WINAPI IsXMPlayVersion(char *text) {
    const char *xmp = "XMPlay";
    int x;
    for(x = 0; x < 6 && *text != (char)NULL; x++, text++) {
        if(*text != xmp[x]) {
            return FALSE;
        }
    }
    while(*text==' ') text++;
    // major version...
    if(*text < '0' || *text > '9') {
        return FALSE;
    }
    while(*text >= '0' && *text <= '9') text++;
    if(*text++ != '.') {
        return FALSE;
    }
    // minor version...
    if(*text < '0' || *text > '9') {
        return FALSE;
    }
    while(*text >= '0' && *text <= '9') text++;
    if(*text == (char)NULL) {
        return TRUE;
    } else {
        return FALSE;
    }
}

static void WINAPI SetNowPlaying(char *text) {
    char *lpTitle;
    int strLen;
    wchar_t *lpMsn;
    HWND msnui = NULL;

    if(text==NULL) {
        lpTitle = "\\0Music\\00\\0{0}\\0\\0";
    } else {
        if(!msnConf.showXMP && IsXMPlayVersion(text)) {
            if(!msnConf.keepOnClose) {
                lpTitle = "\\0Music\\00\\0{0}\\0\\0";
            } else {
                return;
            }
        } else {
            lpTitle = heapAlloc(lstrlenA(text) + 20);
            wsprintfA(lpTitle, "\\0Music\\01\\0{0}\\0%s\\0", text);
        }
    }
    strLen = lstrlenA(lpTitle) + 1;

    lpMsn = heapAlloc(strLen * 2);
    MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, lpTitle, -1, lpMsn, strLen);

    msndata.dwData = 0x547;
    msndata.lpData = (void*)lpMsn;
    msndata.cbData = strLen * 2;

    while(msnui = FindWindowEx(NULL, msnui, "MsnMsgrUIManager", NULL)) {
        SendMessage(msnui, WM_COPYDATA, (WPARAM)NULL, (LPARAM)&msndata);
    }

    if(text != NULL) {
        heapFree(lpTitle);
    }
    heapFree(lpMsn);
}

static void WINAPI DSP_About(HWND win) {
    MessageBox(win,
               "XMPlay to MSN-Now-Playing Plugin\nCopyright 2005 Elliott Sales de Andrade"
               "\n\nContributors:\nSvante Boberg",
               "XMPlay MSN Plugin",
               MB_ICONINFORMATION);
}

static char *WINAPI DSP_GetDescription(void *inst) {
    return dsp.name;
}

static void *WINAPI DSP_New() {
    char *lpTitle;
    int strLen;

    strLen = GetWindowTextLengthA(dsp.xmpwin);
    lpTitle = heapAlloc(strLen + 1);
    GetWindowText(dsp.xmpwin, lpTitle, strLen + 1);
    SetNowPlaying(lpTitle);
    heapFree(lpTitle);

    lpPrevWndProc = (WNDPROC)SetWindowLong(dsp.xmpwin, GWL_WNDPROC, (LONG)WindowProc);

    msnConf.showXMP = TRUE;
    msnConf.keepOnClose = FALSE;

    return (void*)1;
};

static void WINAPI DSP_Free(void *inst) {
    if(!msnConf.keepOnClose) {
        SetNowPlaying(NULL);
    }
    SetWindowLong(dsp.xmpwin, GWL_WNDPROC, (LONG)lpPrevWndProc);
};

static LRESULT WINAPI WindowProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam) {
    if(msg==WM_SETTEXT) {
        SetNowPlaying((char *)lParam);
    }

    return CallWindowProc(lpPrevWndProc, hWnd, msg, wParam, lParam);
}

static void WINAPI DSP_Config(void *inst, HWND win)
{
    DialogBox(ghInstance, (char*)1000, win, &DSPDialogProc);
}

static DWORD WINAPI DSP_GetConfig(void *inst, void *config) {
    memcpy(config, &msnConf, sizeof(msnConf));
    return sizeof(msnConf); // return size of config info
}

static BOOL WINAPI DSP_SetConfig(void *inst, void *config) {
    memcpy(&msnConf, config, sizeof(msnConf));
    return TRUE;
}

#define MESS(id,m,w,l) SendDlgItemMessage(hWnd,id,m,(WPARAM)(w),(LPARAM)(l))

static BOOL CALLBACK DSPDialogProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
    char *lpTitle;
    int strLen;

    switch (msg) {
        case WM_COMMAND:
            switch (LOWORD(wParam)) {
                case IDOK:
                    EndDialog(hWnd, 0);
                    break;
                case 10:
                    msnConf.showXMP = MESS(10, BM_GETCHECK, 0, 0);
                    strLen = GetWindowTextLengthA(dsp.xmpwin);
                    lpTitle = heapAlloc(strLen + 1);
                    GetWindowText(dsp.xmpwin, lpTitle, strLen + 1);
                    SetNowPlaying(lpTitle);
                    heapFree(lpTitle);
                    break;
                case 20:
                    msnConf.keepOnClose = MESS(20, BM_GETCHECK, 0, 0);
                    break;
            }
            break;
        case WM_INITDIALOG:
            MESS(10, BM_SETCHECK, msnConf.showXMP?1:0, 0);
            MESS(20, BM_SETCHECK, msnConf.keepOnClose?1:0, 0);
            return 1;
    }
    return 0;
}

XMPDSP *WINAPI XMPDSP_GetInterface(DWORD face) {
    return &dsp;
}

BOOL WINAPI _DllMainCRTStartup(HANDLE hDLL, DWORD reason, LPVOID reserved) {
    if(reason==DLL_PROCESS_ATTACH) {
        ghInstance=hDLL;
        DisableThreadLibraryCalls(hDLL);
    }
    return 1;
}
